from rest_framework import routers

from .views import PreSelectDeviceRolesViewSet, SearchViewSet

router = routers.DefaultRouter()
router.register('preselectdeviceroles', PreSelectDeviceRolesViewSet)
router.register('search', SearchViewSet, basename='search')

urlpatterns = router.urls


from django.shortcuts import get_object_or_404, render
from django.views.generic import View

class TopologyHomeView(View):
    """
    Show the home page
    """
    def get(self, request):        
        return render(request, 'netbox_topology_views/index.html')


from rest_framework.serializers import ModelSerializer

from dcim.models import DeviceRole

class PreDeviceRoleSerializer(ModelSerializer):

    class Meta:
        model = DeviceRole
        fields = ('id', 'name')
        


from extras.plugins import PluginConfig

class TopologyViewsConfig(PluginConfig):
    name = 'netbox_topology_views'
    verbose_name = 'Topology views'
    description = 'An plugin to render toplogoy maps'
    version = '0.2'
    author = 'Mattijs Vanhaverbeke'
    author_email = 'author@example.com'
    base_url = 'topology-views'
    required_settings = []
    default_settings = {
        'preselected_device_roles': 'Firewall,Router,Distribution Switch,Core Switch,Internal Switch,Access Switch,Server,Storage,Backup,Wireless AP',
        'IGNORE_CABLE_TYPE': 'dcim.poweroutlet,dcim.powerport',
        'device_img': 'access-switch,core-switch,firewall,router,distribution-switch,backup,storage,wan-network,wireless-ap,server,internal-switch,isp-cpe-material,non-racked-devices,power-units'
    }

config = TopologyViewsConfig



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

menu_items = (
    PluginMenuItem(
        link='plugins:netbox_topology_views:home',
        link_text='Topology',
        buttons=()
    ),
)


from setuptools import setup, find_packages


setup(
    name='netbox-topology-views',
    version='0.2',
    description='An NetBox plugin to create Topology maps',
    url='https://github.com/mattieserver/netbox-topology-views',
    author='Mattijs Vanhaverbeke',
    license='Apache 2.0',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
    ],
    project_urls={
        'Source': 'https://github.com/mattieserver/netbox-topology-views',
    },
)



from rest_framework.viewsets import ModelViewSet, ViewSet, ReadOnlyModelViewSet
from rest_framework.decorators import action
from rest_framework.response import Response

from .serializers import PreDeviceRoleSerializer
from django.conf import settings

from utilities.api import IsAuthenticatedOrLoginNotRequired

from dcim.models import  DeviceRole, Device, Cable


class PreSelectDeviceRolesViewSet(ReadOnlyModelViewSet):
    preselected_device_roles_raw = settings.PLUGINS_CONFIG["netbox_topology_views"]["preselected_device_roles"]
    preselected_device_roles = preselected_device_roles_raw.split(",")
    queryset = DeviceRole.objects.filter(name__in=preselected_device_roles)
    serializer_class = PreDeviceRoleSerializer

class SearchViewSet(ViewSet):
    _ignore_model_permissions = True
    permission_classes = [IsAuthenticatedOrLoginNotRequired]

    def _filter(self, site, role, name):
        filter_devices = Device.objects.all()
        if name is not None:
            filter_devices = filter_devices.filter(name__contains=name)
        if site is not None:
            filter_devices = filter_devices.filter(site__id__in=site)
        if role is not None:
            filter_devices = filter_devices.filter(device_role__id__in=role)
        return filter_devices


    @action(detail=False, methods=['post'])
    def search(self, request):
        name = None
        sites = None
        devicerole = None
        if "devicerole" in request.data:
            if request.data["devicerole"]:
                devicerole = request.data["devicerole"]
        if "sites" in request.data:
            if request.data["sites"]:
                sites = request.data["sites"]
        if "name" in request.data:
            if request.data["name"]:
                name = request.data["name"]

        devices = self._filter(sites, devicerole, name)
        nodes = []
        edges = []
        edge_ids = 0
        cable_ids = []
        for device in devices:
            cables = device.get_cables()
            for cable in cables:
                if cable.id not in cable_ids:
                    cable_ids.append(cable.id)
                    edge_ids += 1
                    edge = {}
                    edge["id"] = edge_ids
                    edge["from"] = cable.termination_a.device.id
                    edge["to"] = cable.termination_b.device.id
                    edge["title"] = "Connection between <br> " + cable.termination_a.device.name + " [" + cable.termination_a.name +  "]<br>" + cable.termination_b.device.name + " [" + cable.termination_b.name + "]"
                    edges.append(edge)
            node = {}
            node["id"] = device.id
            node["name"] = device.name
            node["label"] = device.name + device.device_type.display_name
            node["shape"] = 'image'
            if device.device_role.slug in settings.PLUGINS_CONFIG["netbox_topology_views"]["device_img"]:
                node["image"] = '/static/netbox_topology_views/img/'  + device.device_role.slug + ".png"
            else:
                node["image"] = "/static/netbox_topology_views/img/role-unknown.png"

            for device_custom_field in device.custom_field_values.all():
                if device_custom_field.field.name == "coordinates":
                    cords =  device_custom_field.serialized_value.split(";")
                    node["x"] = int(cords[0])
                    node["y"] = int(cords[1])
                    node["physics"] = False
            
            nodes.append(node)

        results = {}
        results["nodes"] = nodes
        results["edges"] = edges

        return Response(results)


from django.urls import path

from . import views


# Define a list of URL patterns to be imported by NetBox. Each pattern maps a URL to
# a specific view so that it can be accessed by users.
urlpatterns = (
    path('', views.TopologyHomeView.as_view(), name='home'),
)
